@@ -1,7 +1,7 @@
 /*****************************************************************************/
 /*  LibreDWG - free implementation of the DWG file format                    */
 /*                                                                           */
-/*  Copyright (C) 2018-2019 Free Software Foundation, Inc.                   */
+/*  Copyright (C) 2018-2020 Free Software Foundation, Inc.                   */
 /*                                                                           */
 /*  This library is free software, licensed under the terms of the GNU       */
 /*  General Public License as published by the Free Software Foundation,     */
@@ -34,6 +34,12 @@
 #include "logging.h"
 #include "suffix.inc"
 
+#ifdef __AFL_COMPILER
+#include "decode.h"
+#include "encode.h"
+#include "in_dxf.h"
+#endif
+
 static int help (void);
 
 static int opts = 1;
@@ -64,7 +70,7 @@ help (void)
   printf ("Default DWGFILE: DXFFILE with .dwg extension in the current "
           "directory.\n"
           "Existing files are not overwritten, unless -y is given.\n"
-          "Encoding currently only works for R13-R2000, dxf2dwg is highly "
+          "Encoding currently only works for R13-R2000, dxf2dwg is "
           "experimental.\n"
           "\n");
 #ifdef HAVE_GETOPT_LONG
@@ -96,7 +102,7 @@ help (void)
   return 0;
 }
 
-// lsan/valgrind leaks still TODO
+// lsan/valgrind leaks still TODO. GH #151
 #if defined __SANITIZE_ADDRESS__ || __has_feature(address_sanitizer)
 const char *__asan_default_options (void);
 const char *
@@ -106,6 +112,57 @@ __asan_default_options (void)
 }
 #endif
 
+#ifdef __AFL_COMPILER
+__AFL_FUZZ_INIT();
+int main (int argc, char *argv[])
+{
+  Dwg_Data dwg;
+  Bit_Chain dat = { NULL, 0, 0, 0, 0 };
+  Bit_Chain out_dat = { NULL, 0, 0, 0, 0 };
+  FILE *fp;
+  struct stat attrib;
+
+  __AFL_INIT();
+  //dat.opts = 3;
+
+  while (__AFL_LOOP(10000)) { // llvm_mode persistent, non-forking mode
+#if 1 // fastest mode via shared mem (10x faster)
+    dat.chain = __AFL_FUZZ_TESTCASE_BUF;
+    dat.size = __AFL_FUZZ_TESTCASE_LEN;
+    printf ("Fuzzing in_dxf + encode from shmem (%lu)\n", dat.size);
+#elif 0 // still 10x faster than the old file-forking fuzzer.
+    /* from stdin: */
+    dat.size = 0;
+    //dat.chain = NULL;
+    dat_read_stream (&dat, stdin);
+    printf ("Fuzzing in_dxf + encode from stdin (%lu)\n", dat.size);
+#else
+    /* else from file */
+    fp = fopen (argv[1], "rb");
+    if (!fp)
+      return 0;
+    dat.size = 0;
+    dat_read_file (&dat, fp, argv[1]);
+    fclose (fp);
+    printf ("Fuzzing in_dxf + encode from file (%lu)\n", dat.size);
+#endif
+
+    if (dat.size < 100) continue;  // useful minimum input length
+    if (dwg_read_dxf (&dat, &dwg) <= DWG_ERR_CRITICAL) {
+      memset (&out_dat, 0, sizeof (out_dat));
+      bit_chain_set_version (&out_dat, &dat);
+      out_dat.version = R_2000;      
+      dwg_encode (&dwg, &out_dat);
+      free (out_dat.chain);
+      dwg_free (&dwg);
+    }
+  }
+  dwg_free (&dwg);
+}
+#define main orig_main
+int orig_main (int argc, char *argv[]);
+#endif
+
 int
 main (int argc, char *argv[])
 {
@@ -116,7 +173,7 @@ main (int argc, char *argv[])
   const char *version = NULL;
   char *filename_out = NULL;
   Dwg_Version_Type dwg_version = R_2000;
-  int do_free;
+  int do_free = 0;
   int need_free = 0;
   int c;
 #ifdef HAVE_GETOPT_LONG
@@ -125,6 +182,7 @@ main (int argc, char *argv[])
       = { { "verbose", 1, &opts, 1 }, // optional
           { "file", 1, 0, 'o' },      { "as", 1, 0, 'a' },
           { "overwrite", 0, 0, 'y' }, { "help", 0, 0, 0 },
+          { "force-free", 0, 0, 0 },
           { "version", 0, 0, 0 },     { NULL, 0, NULL, 0 } };
 #endif
 
@@ -173,6 +231,8 @@ main (int argc, char *argv[])
             return opt_version ();
           if (!strcmp (long_options[option_index].name, "help"))
             return help ();
+          if (!strcmp (long_options[option_index].name, "force-free"))
+            do_free = 1;
           break;
 #else
         case 'i':
@@ -222,12 +282,12 @@ main (int argc, char *argv[])
     }
   i = optind;
 
-  if (filename_out && i + 1 < argc)
+  if (filename_out != NULL && (i + 1) < argc)
     {
       fprintf (stderr, "%s: no -o with multiple input files\n", argv[0]);
       return usage ();
     }
-  do_free = i + 1 < argc;
+  do_free |= (i + 1) < argc; // if more than one file
 
   while (i < argc)
     {
@@ -247,7 +307,8 @@ main (int argc, char *argv[])
         }
 
       dwg.opts = opts;
-      printf ("Warning: dxf2dwg is still highly experimental.\n");
+      dwg.header.version = dwg_version;
+      printf ("Warning: dxf2dwg is still experimental.\n");
       printf ("Reading DXF file %s\n", filename_in);
       error = dxf_read_file (filename_in, &dwg);
       if (error >= DWG_ERR_CRITICAL)
@@ -269,8 +330,9 @@ main (int argc, char *argv[])
         }
       else
         {
-          // FIXME: for now only R_2000. later remove this line.
-          dwg.header.version = dwg_version;
+          // FIXME: for now only R_13 - R_2000. later remove this line.
+          if (dwg.header.from_version < R_13 || dwg.header.from_version >= R_2004)
+            dwg.header.version = dwg_version;
           if (dwg.header.from_version == R_INVALID)
             dwg.header.from_version = dwg.header.version;
           printf ("\n");
@@ -301,6 +363,16 @@ main (int argc, char *argv[])
                     unlink (filename_out);
                     error = dwg_write_file (filename_out, &dwg);
                   }
+                else if (
+#ifdef _WIN32
+      strEQc (filename_out, "NUL")
+#else
+      strEQc (filename_out, "/dev/null")
+#endif
+                         )
+                  {
+                    error = dwg_write_file (filename_out, &dwg);
+                  }
                 else
                   {
                     LOG_ERROR ("Not writable file or symlink: %s\n",
@@ -319,11 +391,18 @@ main (int argc, char *argv[])
       if (error)
         fprintf (stderr, "WRITE ERROR 0x%x %s\n", error, filename_out);
 
-      // forget about valgrind. really huge DWG's need endlessly here.
-      if (do_free
 #if defined __SANITIZE_ADDRESS__ || __has_feature(address_sanitizer)
-          || 0 /* and skip detect_leaks via __asan_default_options() */
+  {
+    char *asanenv = getenv("ASAN_OPTIONS");
+    if (!asanenv)
+      do_free = 1;
+    // detect_leaks is enabled by default. see if it's turned off
+    else if (strstr (asanenv, "detect_leaks=0") == NULL) /* not found */
+      do_free = 1;
+  }
 #endif
+      // forget about leaks. really huge DWG's need endlessly here.
+      if (do_free
 #ifdef HAVE_VALGRIND_VALGRIND_H
           || (RUNNING_ON_VALGRIND)
 #endif
