@@ -258,6 +258,7 @@ char *strrplc (const char *s, const char *from, const char *to);
 #define strEQ(s1, s2) !strcmp ((s1), (s2))
 #define strNE(s1, s2) strcmp ((s1), (s2))
 #define strEQc(s1, s2) !strcmp ((s1), s2 "")
+#define strNEc(s1, s2) strcmp ((s1), s2 "")
 
 #define memBEGIN(s1, s2, len) (strlen (s1) >= len && !memcmp (s1, s2, len))
 #define memBEGINc(s1, s2)                                                     \
@@ -283,8 +284,8 @@ void *memmem (const void *h0, size_t k, const void *n0, size_t l) __nonnull((1,
 // push to handle vector at the end. It really is unshift.
 #define PUSH_HV(_obj, numfield, hvfield, ref)                                 \
   {                                                                           \
-    _obj->hvfield                                                             \
-        = realloc (_obj->hvfield, (_obj->numfield + 1) * sizeof (BITCODE_H)); \
+    _obj->hvfield = (BITCODE_H *)realloc (                                    \
+        _obj->hvfield, (_obj->numfield + 1) * sizeof (BITCODE_H));            \
     _obj->hvfield[_obj->numfield] = ref;                                      \
     LOG_TRACE ("%s[%d] = " FORMAT_REF " [H]\n", #hvfield, _obj->numfield,     \
                ARGS_REF (_obj->hvfield[_obj->numfield]));                     \
@@ -298,4 +299,8 @@ void *memmem (const void *h0, size_t k, const void *n0, size_t l) __nonnull((1,
 #define SHIFT_HV(_obj, numfield, hvfield) shift_hv (_obj->hvfield, &_obj->numfield)
 BITCODE_H shift_hv (BITCODE_H *hv, BITCODE_BL *num_p);
 
+// used in dwg.spec
+Dwg_Handle *dwg_find_first_type_handle (Dwg_Data *restrict dwg,
+                                        enum DWG_OBJECT_TYPE type);
+
 #endif
