@@ -1,7 +1,7 @@
 /*****************************************************************************/
 /*  LibreDWG - free implementation of the DWG file format                    */
 /*                                                                           */
-/*  Copyright (C) 2018-2019 Free Software Foundation, Inc.                   */
+/*  Copyright (C) 2018-2020 Free Software Foundation, Inc.                   */
 /*                                                                           */
 /*  This library is free software, licensed under the terms of the GNU       */
 /*  General Public License as published by the Free Software Foundation,     */
@@ -23,6 +23,9 @@
 #include "decode.h"
 #include "dynapi.h"
 
+EXPORT int dwg_read_dxf (Bit_Chain *restrict dat, Dwg_Data *restrict dwg);
+EXPORT int dwg_read_dxfb (Bit_Chain *restrict dat, Dwg_Data *restrict dwg);
+
 // global array of [obj -> [fields], ...]
 typedef struct _dxf_field
 {
@@ -43,14 +46,14 @@ typedef struct _dxf_objs
 typedef struct _dxf_pair
 {
   short code;
-  enum RES_BUF_VALUE_TYPE type;
+  enum RESBUF_VALUE_TYPE type;
   union // must be big enough for setting BD
   {
     unsigned int u;
     int i;
     char *s;
     long l;
-    uint64_t bll;
+    uint64_t rll;
     double d;
   } value;
 } Dxf_Pair;
@@ -74,13 +77,10 @@ typedef struct _array_hdls
   struct array_hdl items[]; // Flexible array grows
 } array_hdls;
 
-#define ARRAY_SIZE(arr) (sizeof (arr) / sizeof (arr[0]))
-
 array_hdls *array_push (array_hdls *restrict hdls, const char *restrict field,
                         const char *restrict name, const int code);
 array_hdls *new_array_hdls (int size);
 void free_array_hdls (array_hdls *hdls);
-int matches_type (Dxf_Pair *restrict pair, const Dwg_DYNAPI_field *restrict f);
 
 void dxf_add_field (Dwg_Object *restrict obj, const char *restrict name,
                     const char *restrict type, int dxf);
@@ -88,24 +88,13 @@ Dxf_Field *dxf_search_field (Dwg_Object *restrict obj,
                              const char *restrict name,
                              const char *restrict type, int dxf);
 
-void add_eed (Dwg_Object *restrict obj, const char *restrict name,
-              Dxf_Pair *restrict pair);
-Dxf_Pair *add_xdata (Bit_Chain *restrict dat, Dwg_Object *restrict obj,
-                     Dxf_Pair *restrict pair);
-void add_dictionary_itemhandles (Dwg_Object *restrict obj,
-                                 Dxf_Pair *restrict pair, char *restrict text);
-void resolve_postponed_header_refs (Dwg_Data *restrict dwg);
-void resolve_postponed_object_refs (Dwg_Data *restrict dwg);
-void resolve_postponed_eed_refs (Dwg_Data *restrict dwg);
-void resolve_header_dicts (Dwg_Data *restrict dwg);
 BITCODE_H find_tablehandle (Dwg_Data *restrict dwg, Dxf_Pair *restrict pair);
-int is_table_name (const char *name);
-void entity_alias (char *name);
-void object_alias (char *name);
-Dwg_Object *find_prev_entity (Dwg_Object *obj);
+int is_table_name (const char *restrict name);
+int is_textlike (Dwg_Object *restrict obj);
+void in_postprocess_handles (Dwg_Object *restrict obj);
+void in_postprocess_SEQEND (Dwg_Object *restrict obj, BITCODE_BL num_owned, BITCODE_H *owned);
 
-EXPORT int dwg_read_dxf (Bit_Chain *restrict dat, Dwg_Data *restrict dwg);
-EXPORT int dwg_read_dxfb (Bit_Chain *restrict dat, Dwg_Data *restrict dwg);
+BITCODE_RC dxf_find_lweight (const int lw);
 
 // for sscanf with BD we need to use %lf not %g
 #undef FORMAT_BD
@@ -120,7 +109,8 @@ EXPORT int dwg_read_dxfb (Bit_Chain *restrict dat, Dwg_Data *restrict dwg);
     (void)dwg_add_object (dwg);                                               \
     obj = &dwg->object[idx];                                                  \
     obj->supertype = DWG_SUPERTYPE_OBJECT;                                    \
-    obj->tio.object = calloc (1, sizeof (Dwg_Object_Object));                 \
+    obj->tio.object                                                           \
+        = (Dwg_Object_Object *)calloc (1, sizeof (Dwg_Object_Object));        \
     obj->tio.object->objid = obj->index;                                      \
     obj->tio.object->dwg = dwg;                                               \
   }
@@ -131,7 +121,8 @@ EXPORT int dwg_read_dxfb (Bit_Chain *restrict dat, Dwg_Data *restrict dwg);
     (void)dwg_add_object (dwg);                                               \
     obj = &dwg->object[idx];                                                  \
     obj->supertype = DWG_SUPERTYPE_ENTITY;                                    \
-    obj->tio.entity = calloc (1, sizeof (Dwg_Object_Entity));                 \
+    obj->tio.entity                                                           \
+        = (Dwg_Object_Entity *)calloc (1, sizeof (Dwg_Object_Entity));        \
     obj->tio.entity->objid = obj->index;                                      \
     obj->tio.entity->dwg = dwg;                                               \
   }
@@ -169,8 +160,35 @@ EXPORT int dwg_read_dxfb (Bit_Chain *restrict dat, Dwg_Data *restrict dwg);
       ADD_OBJECT (token);                                                     \
     }
 
-#define STRADD(field, string)                                                 \
-  field = malloc (strlen (string) + 1);                                       \
-  strcpy (field, string)
+#define STRADD_TV(field, string)                                              \
+  if (string)                                                                 \
+    {                                                                         \
+      field = (char*)malloc (strlen (string) + 1);                            \
+      strcpy (field, string);                                                 \
+    }
+#define STRADD_T(field, string)                                               \
+  if (string)                                                                 \
+    {                                                                         \
+      if (dat->version >= R_2007)                                             \
+        field = (char*)bit_utf8_to_TU (string);                               \
+      else                                                                    \
+        {                                                                     \
+          field = (char*)malloc (strlen (string) + 1);                        \
+          strcpy (field, string);                                             \
+        }                                                                     \
+    }
+
+#define UPGRADE_ENTITY(FROM, TO)                                              \
+  obj->type = obj->fixedtype = DWG_TYPE_##TO;                                 \
+  obj->name = (char *)#TO;                                                    \
+  obj->dxfname = strdup (obj->name);                                          \
+  strcpy (name, obj->name);                                                   \
+  LOG_TRACE ("change type to %s\n", name);                                    \
+  if (sizeof (Dwg_Entity_##TO) > sizeof (Dwg_Entity_##FROM))                  \
+    {                                                                         \
+      LOG_TRACE ("realloc to %s\n", name);                                    \
+      _obj = realloc (_obj, sizeof (Dwg_Entity_##TO));                        \
+      obj->tio.entity->tio.TO = (Dwg_Entity_##TO *)_obj;                      \
+    }
 
 #endif
