@@ -13,6 +13,7 @@
 #ifndef SPEC_H
 #  define SPEC_H
 
+#  undef _GNU_SOURCE
 #  define _GNU_SOURCE
 #  include <string.h>
 #  include "decode.h"
@@ -351,18 +352,24 @@
 #endif
 // on DXF skip if 0
 #ifndef FIELD_BD0
+#  define FIELD_2RD0(name, dxf) FIELD_2RD (name, dxf)
 #  define FIELD_RD0(name, dxf) FIELD_RD (name, dxf)
+#  define FIELD_RD1(name, dxf) FIELD_RD (name, dxf)
 #  define FIELD_BD0(name, dxf) FIELD_BD (name, dxf)
 #  define FIELD_BD1(name, dxf) FIELD_BD (name, dxf)
 #  define FIELD_BL0(name, dxf) FIELD_BL (name, dxf)
 #  define SUB_FIELD_BL0(o, name, dxf) SUB_FIELD_BL (o, name, dxf)
 #  define FIELD_B0(name, dxf) FIELD_B (name, dxf)
 #  define FIELD_BS0(name, dxf) FIELD_BS (name, dxf)
+#  define FIELD_BS1(name, dxf) FIELD_BS (name, dxf)
 #  define FIELD_RC0(name, dxf) FIELD_RC (name, dxf)
 #  define FIELD_RS0(name, dxf) FIELD_RS (name, dxf)
 #  define FIELD_RL0(name, dxf) FIELD_RL (name, dxf)
 #  define FIELD_BT0(name, dxf) FIELD_BT (name, dxf)
+#  define VALUE_T0(name, dxf) VALUE_T (name, dxf)
+#  define FIELD_TV0(name, dxf) FIELD_TV (name, dxf)
 #  define FIELD_T0(name, dxf) FIELD_T (name, dxf)
+#  define FIELD_CMC0(color, dxf) FIELD_CMC (color, dxf)
 #  define FIELD_HANDLE0(name, code, dxf) FIELD_HANDLE (name, code, dxf)
 #  define SUB_FIELD_HANDLE0(o, name, code, dxf) SUB_FIELD_HANDLE (o, name, code, dxf)
 #endif
