@@ -2,12 +2,13 @@
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
+#include <stddef.h>
 #include <stdarg.h>
 #if defined(BITS_TEST_C) || defined(DECODE_TEST_C)
 #  include "../../src/bits.h"
 #endif
 
-#if defined(_WIN32) && defined HAVE_FUNC_ATTRIBUTE_MS_FORMAT
+#if defined(_WIN32) && defined(HAVE_FUNC_ATTRIBUTE_MS_FORMAT) && !defined(__USE_MINGW_ANSI_STDIO)
 #  define ATTRIBUTE_FORMAT(x, y) __attribute__ ((format (ms_printf, x, y)))
 #elif defined HAVE_FUNC_ATTRIBUTE_GNU_FORMAT
 #  define ATTRIBUTE_FORMAT(x, y) __attribute__ ((format (gnu_printf, x, y)))
@@ -26,9 +27,7 @@ int numpassed (void);
 int numfailed (void);
 static inline void pass (void);
 static void fail (const char *fmt, ...) ATTRIBUTE_FORMAT (1, 2);
-#ifndef DYNAPI_TEST_C
 static void ok (const char *fmt, ...) ATTRIBUTE_FORMAT (1, 2);
-#endif
 
 int
 numpassed (void)
@@ -41,8 +40,8 @@ numfailed (void)
   return failed;
 }
 
-#ifndef DYNAPI_TEST_C
 static void
+ATTRIBUTE_FORMAT (1, 2)
 ok (const char *fmt, ...)
 {
   va_list ap;
@@ -53,7 +52,6 @@ ok (const char *fmt, ...)
   va_end (ap);
   printf ("ok %d\t# %s\n", ++num, buffer);
 }
-#endif
 
 static inline void
 pass (void)
@@ -63,6 +61,7 @@ pass (void)
 }
 
 static void
+ATTRIBUTE_FORMAT (1, 2)
 fail (const char *fmt, ...)
 {
   va_list ap;
@@ -93,9 +92,9 @@ bitprepare (Bit_Chain *dat, size_t size)
   dat->byte = 0;
   dat->from_version = dat->version = R_2000;
   dat->opts = 1;
-  dat->size = size;
+  dat->size = size + 1;
   dat->fh = NULL;
-  dat->chain = (unsigned char *)calloc (size, 1);
+  dat->chain = (unsigned char *)calloc (size + 1, 1);
 }
 
 /*
